button[data-v-2c15ccb6]{display:flex;align-items:center;gap:.5rem;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;border-radius:.5rem;padding:.25rem .75rem}.icon[data-v-2c15ccb6]{height:1rem;width:1rem}
