import{o as e,a as s,b as i,q as l}from"./index-mtICtCH6.js";const t={__name:"SLLoading",props:{size:{type:String,default:"8"}},setup:t=>(a,d)=>(e(),s("div",{class:l(["relative flex items-center justify-center rounded-full overflow-hidden animate-spin",t.size?`w-${t.size} h-${t.size}`:"w-8 h-8"])},d[0]||(d[0]=[i("div",{class:"absolute w-full h-full rounded-full border-4 border-t-positive-default border-positive-dimmest"},null,-1),i("div",{class:"w-2 h-2 z-10 rounded-full bg-positive-default animate-ping"},null,-1)]),2))};export{t as _};
