import{_ as e,I as n,o as a,a as s,F as t,s as r,e as o,w as i,b as l,t as c,A as u,q as d,d as h}from"./index-mtICtCH6.js";import{u as m}from"./experiment-C9fiIMV8.js";const p={class:"lg:w-48 lg:h-auto lg:px-4 lg:transition-none px-6 w-0 h-0 transition-all overflow-hidden"},v=e({__name:"SiderNav",setup(e){const d=[{key:"index",to:"index",icon:"home"},{key:"hardware",to:"hardware",icon:"config"},{key:"requirements",to:"requirements",icon:"help"}];return(e,h)=>{const m=u,v=n("router-link");return a(),s("div",null,[(a(),s(t,null,r(d,(n=>o(v,{key:n.key,to:n.to,"active-class":"active-router"},{default:i((()=>[l("span",p,c(e.$t(`experiment.env.navs.${n.key}`)),1),o(m,{icon:n.icon,class:"w-4 h-4 lg:w-0 lg:h-0 transition-all absolute left-4"},null,8,["icon"])])),_:2},1032,["to"]))),64))])}}},[["__scopeId","data-v-2a435598"]]),f={class:"w-full rounded-lg mr-6"},x={__name:"EnvironmentPage",setup(e){const t=m();return(e,r)=>{const i=n("RouterView");return a(),s("div",{class:d(["w-full h-full flex p-6 bg-higher",h(t).experiment.description?"min-h-[calc(100vh-195px)]":"min-h-[calc(100vh-163px)]"])},[o(v,{class:"shrink-0 mr-6"}),l("div",f,[o(i)])],2)}}};export{x as default};
