.input[data-v-052f7e40]{width:100%;border-radius:.5rem;border-width:1px;background-color:var(--background-higher, #f6f8fA);padding:.5rem;font-size:.875rem;line-height:1.25rem;outline:2px solid transparent;outline-offset:2px}.tip[data-v-052f7e40]{position:absolute;bottom:-20px;left:0;font-size:.75rem;line-height:1rem}button[type=submit][data-v-052f7e40]:disabled{pointer-events:none;cursor:not-allowed;gap:.5rem;opacity:.5}button[data-v-41907065]{display:flex;height:2rem;align-items:center;border-radius:.5rem;border-style:none;padding-left:.75rem;padding-right:.75rem;border-color:var(--outline-default)!important}@media (min-width: 768px){button[data-v-41907065]{border-style:solid}}.sl-status-label[data-v-b6e5701f]{display:inline-flex;align-items:center;gap:.375rem;border-radius:9999px;padding:.25rem .75rem;text-align:center;font-size:.875rem;line-height:1.25rem}.crashed[data-v-b6e5701f]{background-color:var(--negative-dimmest, #ff9494);color:var(--negative-default, #fa4b4b)}.crashed svg[data-v-b6e5701f]{height:1rem;width:1rem}.finished[data-v-b6e5701f]{background-color:var(--positive-dimmest, #97e896);color:var(--positive-higher, #036e15)}.finished svg[data-v-b6e5701f]{height:1rem;width:1rem}.running[data-v-b6e5701f]{background-color:var(--primary-dimmest, #b2d9ff);color:var(--primary-higher, #005cb8)}.running svg[data-v-b6e5701f]{height:1rem;width:1rem}@keyframes breathe-b6e5701f{0%{transform:scale(.8);opacity:.5}50%{transform:scale(1);opacity:1}to{transform:scale(.8);opacity:.5}}.running svg[data-v-b6e5701f]{animation:breathe-b6e5701f 1.5s ease-in-out infinite}
