import{u as e}from"./experiment-C9fiIMV8.js";import{u as t,c as a,a as l,b as s,t as n,g as r,f as i,o as p}from"./index-mtICtCH6.js";import{E as u,g as o}from"./parser-YUbCvSuN.js";const m={class:"w-full border p-6 rounded-lg bg-default"},d={class:"w-full text-xl font-semibold pb-4 border-b"},v={__name:"EnvIndex",setup(v){const{experiment:x}=e(),g=t(),f=a((()=>o(x,g)));return(e,t)=>(p(),l("div",m,[s("h1",d,n(e.$t("experiment.env.title.overview")),1),f.value.experiment?(p(),r(u,{key:0,title:e.$t("experiment.env.groups.experiment"),data:f.value.experiment},null,8,["title","data"])):i("",!0),f.value.system?(p(),r(u,{key:1,title:e.$t("experiment.env.groups.system"),data:f.value.system},null,8,["title","data"])):i("",!0),f.value.python?(p(),r(u,{key:2,title:e.$t("experiment.env.groups.python"),data:f.value.python},null,8,["title","data"])):i("",!0),f.value.git?(p(),r(u,{key:3,title:e.$t("experiment.env.groups.git"),data:f.value.git},null,8,["title","data"])):i("",!0)]))}};export{v as default};
