.log-container[data-v-0942f820]{position:relative;height:76vh;width:100%;overflow:auto;border-radius:.25rem;background-color:var(--background-higher, #f6f8fA);padding:1rem;font-size:13px;line-height:16px;font-family:JetBrains Mono,monospace;letter-spacing:.1px;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.log-container .log-area[data-v-0942f820]{position:relative;height:100%;word-break:break-all}.log-container .log-area[data-v-0942f820]::-webkit-scrollbar-track{background:transparent}.log-container .log-area .log-board[data-v-0942f820]{position:absolute;width:100%;padding-bottom:1.25rem}.log-container .log-line[data-v-0942f820]{display:flex;gap:.5rem;white-space:pre-wrap}.magnifier[data-v-0942f820]{height:2.5rem;width:2.5rem;animation:animloader-0942f820 3s infinite}@keyframes animloader-0942f820{0%{transform:translate(-5px,-5px)}25%{transform:translate(-5px,5px)}50%{transform:translate(5px,5px)}75%{transform:translate(5px,-5px)}to{transform:translate(-5px,-5px)}}
