#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024-01-18 13:08:48
@File: swanlab/db/models/__init__.py
@IDE: vscode
@Description:
    在此处自定义模型类，继承自SwanModel
"""
from .charts import Chart
from .displays import Display
from .experiments import Experiment
from .namespaces import Namespace
from .projects import Project
from .tags import Tag
from .sources import Source
