.item[data-v-0bf7a51b]{left:12rem;word-break:break-all}.item:hover .copy-button[data-v-0bf7a51b]{opacity:1}.item .copy-button[data-v-0bf7a51b]{position:absolute;top:.375rem;right:.25rem;height:1rem;width:1rem;opacity:0;transition-property:all;transition-timing-function:cubic-bezier(.4,0,.2,1);transition-duration:.15s}.high-light[data-v-0bf7a51b]{background-color:var(--background-higher, #f6f8fA);padding-left:.5rem;padding-right:.5rem;color:var(--foreground-dimmer, #555)}.copy[data-v-0bf7a51b]{transition-property:all;transition-timing-function:cubic-bezier(.4,0,.2,1);transition-duration:.15s}.copy[data-v-0bf7a51b]:hover{--tw-scale-y: 1;transform:translate(var(--tw-translate-x),var(--tw-translate-y)) rotate(var(--tw-rotate)) skew(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));cursor:pointer;border-radius:.25rem;background-color:var(--background-highest, #dadde1);padding-right:2rem;padding-left:.5rem}.env-items[data-v-167a6b3c]{width:100%;flex-direction:column}.env-items[data-v-167a6b3c]>:not([hidden])~:not([hidden]){--tw-space-y-reverse: 0;margin-top:calc(1.25rem * calc(1 - var(--tw-space-y-reverse)));margin-bottom:calc(1.25rem * var(--tw-space-y-reverse))}.env-items[data-v-167a6b3c]{padding-top:1.25rem;padding-bottom:1.25rem}.env-items[data-v-167a6b3c]:not(:last-child){border-bottom-width:1px}
