.icon[data-v-e7e0a975]{height:1.25rem;width:1.25rem;cursor:pointer;color:var(--foreground-dimmest, #b5b5b5)}.icon[data-v-e7e0a975]:hover{color:var(--foreground-dimmer, #555)}.title[data-v-e7e0a975]{min-width:150px;color:var(--foreground-dimmest, #b5b5b5)}.table[data-v-7df30f46]{width:100%;border-radius:.5rem;outline-style:solid;outline-color:var(--outline-default);outline-width:1px}.line[data-v-7df30f46]{display:grid;grid-template-columns:repeat(5,minmax(0,1fr));gap:.75rem}.data-line[data-v-7df30f46]{border-bottom-width:1px;border-color:var(--outline-dimmest, #f2f2f2);display:grid;grid-template-columns:repeat(5,minmax(0,1fr));gap:.75rem}.data-line[data-v-7df30f46]:last-child{border-style:none}.data-line[data-v-7df30f46]:hover{background-color:var(--background-higher, #f6f8fA)}.data-line[data-v-7df30f46]:last-child{border-bottom-right-radius:.5rem;border-bottom-left-radius:.5rem}.header-item[data-v-7df30f46]{margin-top:.75rem;margin-bottom:.75rem;border-right-width:1px;padding-left:1rem}.header-item[data-v-7df30f46]:first-child{grid-column:span 2 / span 2}.header-item[data-v-7df30f46]:last-child{grid-column:span 3 / span 3;border-style:none;padding-left:0}.body-item[data-v-7df30f46]{overflow-wrap:break-word;padding:.75rem .5rem .75rem 1rem}.body-item[data-v-7df30f46]:first-child{grid-column:span 2 / span 2}.body-item[data-v-7df30f46]:last-child{grid-column:span 3 / span 3;border-style:none;padding-left:0}
