import{a as e,b as s,t,e as a,w as o,B as r,o as l,D as d}from"./index-mtICtCH6.js";const i={class:"w-full flex flex-col items-center pt-10"},m={class:"py-4 text-dimmer font-semibold text-lg"},n={__name:"NotFound",setup:n=>(n,p)=>(l(),e("div",i,[p[1]||(p[1]=s("h1",{class:"text-5xl font-semibold"},"404",-1)),s("p",m,t(n.$t("error.404.message")),1),a(r,{class:"px-4 py-1.5 border rounded-lg",onClick:p[0]||(p[0]=e=>n.$router.replace("/")),theme:"positive"},{default:o((()=>[d(t(n.$t("error.404.return")),1)])),_:1})]))};export{n as default};
