@charset "UTF-8";.experiment-title[data-v-941ebc81]{display:flex;width:100%;align-items:center;overflow-x:auto;overflow-y:visible}.experiment-title[data-v-941ebc81]::-webkit-scrollbar{display:none}.experiment-description[data-v-941ebc81]{margin-top:.75rem;width:100%;overflow-wrap:break-word;font-size:.875rem;line-height:1.25rem;color:var(--foreground-dimmer, #555);display:-webkit-box;-webkit-box-orient:vertical;overflow:hidden;-webkit-line-clamp:2}.experiment-navs[data-v-941ebc81]{margin-top:.75rem;display:flex;width:100%;align-items:center;gap:2rem;overflow-x:auto;overflow-y:visible}.experiment-navs[data-v-941ebc81]::-webkit-scrollbar{display:none}.experiment-navs .nav-item[data-v-941ebc81]{position:relative;white-space:nowrap;padding:.5rem .25rem .375rem;color:var(--foreground-dimmer, #555);outline:2px solid transparent;outline-offset:2px;--tw-ring-offset-shadow: var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color);--tw-ring-shadow: var(--tw-ring-inset) 0 0 0 calc(0px + var(--tw-ring-offset-width)) var(--tw-ring-color);box-shadow:var(--tw-ring-offset-shadow),var(--tw-ring-shadow),var(--tw-shadow, 0 0 #0000);margin-bottom:.25rem;display:flex;align-items:center;gap:.375rem;border-radius:.25rem}.experiment-navs .nav-item[data-v-941ebc81]:hover{background-color:var(--background-higher, #f6f8fA)}.experiment-navs .nav-active[data-v-941ebc81]{font-weight:600;color:var(--foreground-default, #000)}.experiment-navs .nav-active[data-v-941ebc81]:hover{background-color:transparent!important}.experiment-navs .nav-active[data-v-941ebc81]:after{position:absolute;bottom:-.25rem;left:50%;z-index:10;height:.125rem;width:100%;--tw-translate-x: -50%;transform:translate(var(--tw-translate-x),var(--tw-translate-y)) rotate(var(--tw-rotate)) skew(var(--tw-skew-x)) skewY(var(--tw-skew-y)) scaleX(var(--tw-scale-x)) scaleY(var(--tw-scale-y));background-color:var(--positive-higher, #036e15);content:""}
