import{aq as e,i as t,c as a,x as n}from"./index-mtICtCH6.js";const r=e("experiment",(()=>{const e=t(),r=t(),u=a((()=>{var t;return null==(t=e.value)?void 0:t.id})),i=a((()=>{var t;return null==(t=e.value)?void 0:t.name})),l=a((()=>{var t;return null==(t=e.value)?void 0:t.description})),s=a((()=>{var t;return null==(t=e.value)?void 0:t.status})),v=a((()=>{var t;return null==(t=e.value)?void 0:t.light})),o=a((()=>0===s.value)),d=a((()=>n(e.value)||""));return{experiment:e,charts:r,id:u,name:i,description:l,status:s,color:v,isRunning:o,duration:d,setStatus:t=>{e.value.status=t},setFinishTime:t=>{e.value.finish_time=t},setExperiment:t=>{e.value=t},setExperimentPartial:t=>{for(const a in t)null!=e.value[a]&&(e.value[a]=t[a])}}}));export{r as u};
