import{u as e}from"./experiment-C9fiIMV8.js";import{a,E as l,b as t}from"./parser-YUbCvSuN.js";import{o as u,a as n,b as r,t as p,e as v,F as s,s as d,c as i,g as m,f as o}from"./index-mtICtCH6.js";const y={class:"w-full py-5 flex-col space-y-5"},c={class:"flex items-center gap-2 font-semibold"},f={class:"text-dimmer w-44 shrink-0"},g={class:"max-w-96 w-full break-words whitespace-wrap shrink"},x={class:"shrink-0 pl-8"},k={__name:"NvidiaGpu",props:{data:{type:{driver:String,cuda:String,cores:Number,memory:Array,type:Array},default:()=>({})},type:{type:String,default:"gpu"}},setup:e=>(l,t)=>(u(),n("div",y,[r("h1",c,p(l.$t(`experiment.env.groups.${e.type}`)),1),v(a,{"env-key":"gpu_driver","env-value":e.data.driver},null,8,["env-value"]),v(a,{"env-key":"cuda","env-value":e.data.cuda},null,8,["env-value"]),v(a,{"env-key":"gpu_cores","env-value":e.data.cores},null,8,["env-value"]),(u(!0),n(s,null,d(e.data.memory,((a,l)=>{var t;return u(),n("div",{class:"w-full flex",key:l},[r("span",f,"GPU"+p(l),1),r("span",g,p(null==(t=e.data.type)?void 0:t[l]),1),r("span",x,p(a)+" GB",1)])})),128))]))},w={class:"w-full py-5 flex-col space-y-5"},b={class:"flex items-center gap-2 font-semibold"},h={__name:"AppleChip",props:{data:{type:{type:String,cpu:String,gpu:String,memory:String},default:{}}},setup:e=>(l,t)=>(u(),n("div",w,[r("h1",b,p(l.$t("experiment.env.groups.apple")),1),v(a,{"env-key":"brand","env-value":e.data.type},null,8,["env-value"]),v(a,{"env-key":"cpu","env-value":e.data.cpu},null,8,["env-value"]),v(a,{"env-key":"memory","env-value":e.data.memory+"GB"},null,8,["env-value"])]))},_={key:0,class:"w-full border p-6 rounded-lg bg-default"},S={class:"w-full text-xl font-semibold pb-4 border-b"},$={key:1},j={class:"text-center pt-5"},G={__name:"EnvHardware",setup(a){const{experiment:v}=e(),s=i((()=>t(v))),d=i((()=>{var e;return null==(e=s.value)?void 0:e.apple})),y=e=>!!e&&(null==e?void 0:e.filter((e=>e.value)).length)>0,c=i((()=>y(s.value.cpu))),f=i((()=>y(s.value.memory))),g=i((()=>s.value.apple||c.value||f.value||s.value.gpu));return(e,a)=>g.value?(u(),n("div",_,[r("h1",S,p(e.$t("experiment.env.title.hardware")),1),d.value?(u(),m(h,{key:0,data:s.value.apple},null,8,["data"])):o("",!0),c.value&&!d.value?(u(),m(l,{key:1,title:e.$t("experiment.env.groups.cpu"),data:s.value.cpu},null,8,["title","data"])):o("",!0),f.value&&!d.value?(u(),m(l,{key:2,data:s.value.memory},null,8,["data"])):o("",!0),s.value.nvidia?(u(),m(k,{key:3,data:s.value.nvidia,type:"nvidia"},null,8,["data"])):s.value.gpu?(u(),m(k,{key:4,data:s.value.gpu,type:"gpu"},null,8,["data"])):o("",!0)])):(u(),n("div",$,[r("p",j,p(e.$t("experiment.env.empty.hardware")),1)]))}};export{G as default};
