@charset "UTF-8";.project-title[data-v-c54d48e0]{display:flex;width:100%;align-items:center;overflow-x:auto}.project-title[data-v-c54d48e0]::-webkit-scrollbar{display:none}.project-description[data-v-c54d48e0]{margin-top:.875rem;width:100%;overflow-wrap:break-word;font-size:.875rem;line-height:1.25rem;color:var(--foreground-dimmer, #555);display:-webkit-box;-webkit-box-orient:vertical;overflow:hidden;-webkit-line-clamp:2}.gradient-last-line .line:last-child .cell[data-v-1c6c498d]:first-child:after{content:"";box-sizing:content-box;position:absolute;top:100%;left:0;height:40px;width:100%;border-right:1px;border-style:solid;-o-border-image:linear-gradient(to bottom,var(--outline-default),transparent) 1;border-image:linear-gradient(to bottom,var(--outline-default),transparent) 1}.table-border .table-header[data-v-1c6c498d],.table-border .line[data-v-1c6c498d]{border-left-width:1px;border-right-width:1px}.table-border .line[data-v-1c6c498d]:last-child{border-bottom-width:1px}.cell[data-v-1c6c498d]{flex-shrink:0;white-space:nowrap;text-align:left;box-sizing:border-box;height:54px}.cell[data-v-1c6c498d]:first-child{position:sticky;left:0;z-index:10;border-right-width:1px}.cell[data-v-1c6c498d]:not(:first-child){position:relative;z-index:0}.line[data-v-1c6c498d]{display:flex;align-items:center}.line:hover .cell[data-v-1c6c498d]:not(:hover){background-color:var(--background-higher)}.table-header[data-v-1c6c498d]{position:relative}.table-header .table-header-item:hover button[data-v-1c6c498d]{background-color:var(--foreground-dimmest)}.table-header .table-header-item button[data-v-1c6c498d]{position:absolute;right:0;top:0;height:100%;width:.25rem;cursor:col-resize}.table-header .table-header-item button[data-v-1c6c498d]:hover{width:.5rem;background-color:var(--primary-dimmer, #74b9ff)}.table-header .table-header-item button[data-v-1c6c498d]:active{background-color:var(--primary-dimmer, #74b9ff)}.table-header .table-header-item button[data-v-1c6c498d]{transition-property:all;transition-timing-function:cubic-bezier(.4,0,.2,1);transition-duration:.3s}.button-hover-tip[data-v-1c6c498d]{background-color:var(--foreground-dimmest)}.loader[data-v-bcdae545]{width:106px;height:56px;display:block;margin:30px auto;background-image:linear-gradient(#333 50px,transparent 0),linear-gradient(#333 50px,transparent 0),linear-gradient(#333 50px,transparent 0),linear-gradient(#333 50px,transparent 0),radial-gradient(circle 14px,#333 100%,transparent 0);background-size:48px 15px,15px 35px,15px 35px,25px 15px,28px 28px;background-position:25px 5px,58px 20px,25px 17px,2px 37px,76px 0px;background-repeat:no-repeat;position:relative;transform:rotate(-45deg);box-sizing:border-box}.loader[data-v-bcdae545]:after,.loader[data-v-bcdae545]:before{content:"";position:absolute;width:56px;height:56px;border:6px solid #333;border-radius:50%;left:-45px;top:-10px;background-repeat:no-repeat;background-image:linear-gradient(#333 64px,transparent 0),linear-gradient(#333 66px,transparent 0),radial-gradient(circle 4px,#333 100%,transparent 0);background-size:40px 1px,1px 40px,8px 8px;background-position:center center;box-sizing:border-box;animation:rotation-bcdae545 .3s linear infinite}.loader[data-v-bcdae545]:before{left:25px;top:60px}@keyframes rotation-bcdae545{0%{transform:rotate(0)}to{transform:rotate(360deg)}}input[type=checkbox][data-v-b757f214]{height:1rem;width:1rem;cursor:pointer;border-width:1px}input[type=checkbox][data-v-b757f214]:disabled{cursor:not-allowed}.dashboard-table[data-v-e66567d7]{border-left-width:0px;border-right-width:0px;border-bottom-width:0px}.grow[data-v-e66567d7]{color:var(--foreground-dimmest, #b5b5b5)}@keyframes skeleton-ani-e66567d7{0%{left:0}to{left:100%}}.skeleton[data-v-e66567d7]{background-image:linear-gradient(-45deg,#f5f5f5 40%,#fff 55%,#f5f5f5 63%);list-style:none;background-size:400% 100%;background-position:100% 50%;animation:skeleton-animation-e66567d7 2s ease infinite}@keyframes skeleton-animation-e66567d7{0%{background-position:100% 50%}to{background-position:0 50%}}
