a[data-v-2a435598]{position:relative;margin-bottom:.5rem;display:flex;height:2.75rem;align-items:center;border-radius:.5rem;color:var(--foreground-default, #000)}a[data-v-2a435598]:hover{background-color:var(--background-highest, #dadde1)}.active-router[data-v-2a435598]{background-color:var(--background-highest, #dadde1);font-weight:600}.active-router[data-v-2a435598]:hover{background-color:var(--background-highest, #dadde1)}
